/**
 * \file
 *
 */

#ifdef TRACE

/* awfull hack */
extern int DEBUG_indent;
extern int DEBUG_i;
#define return        DEBUG_indent -= 2;return 

#endif /* TRACE */


extern char * global_errmsg;

/* Man Machine Protocol defs */

/** \brief SCOP Message identification string */
#define SCOP_MAGIC    "SCOP/"
/** \brief SCOP version */
#define SCOP_VERSION  "1.0"

/** \brief SCOP message type */
#define SCOP_INIT_IUT_RQ   "ScopInitRequest"
/** \brief SCOP message type */
#define SCOP_RESET_RQ      "ScopResetRequest"
/** \brief SCOP message type */
#define SCOP_DATA_RQ       "ScopDataRequest"
/** \brief SCOP message type */
#define SCOP_RESPONSE      "ScopResponse"
/** \brief SCOP message type */
#define SCOP_DATA_RESPONSE "ScopDataResponse"
/** \brief SCOP DataRequest command */
#define SCOP_SEND_VIA_IUT  "sendViaIut"
/** \brief SCOP DataRequest command */
#define SCOP_GET_VIA_IUT   "getViaIut"


/* Errors */
/** \brief SCOP error message */
#define SCOP_DECODE_BADLENGTH_ERRMSG        "Bad length"
/** \brief SCOP error message */
#define SCOP_DECODE_TRUNCMSG_ERRMSG         "Message truncated"
/** \brief SCOP error message */
#define SCOP_DECODE_NOTSCOP_ERRMSG          "Message is not a SCOP message"
/** \brief SCOP error message */
#define SCOP_DECODE_BADSCOPVERSION_ERRMSG   "Bad SCOP version"
/** \brief SCOP error message */
#define SCOP_DECODE_BADTYPE_ERRMSG          "Bad message type"
/** \brief SCOP error message */
#define SCOP_DECODE_BADPARAM_ERRMSG         "Unknown parameter"
/** \brief SCOP error message */
#define SCOP_DECODE_BADVALUE_ERRMSG         "Bad value"
/** \brief SCOP error message */
#define SCOP_DECODE_PARAMREPEAT_ERRMSG      "Multiple occurence of the same parameter"
/** \brief SCOP error message */
#define SCOP_DECODE_MISSINGPARAM_ERRMSG     "At least one important parameter is missing"
/** \brief SCOP error message */
#define SCOP_BADCMD_ERRMSG                  "Bad command name"
/** \brief SCOP error message */
#define SCOP_CONFIG_LOADFILE_ERRMSG         "Unable to load configuration file"
/** \brief SCOP error message */
#define SCOP_OK_ERRMSG                      "Ok"

/* Error codes (10 ->29) */
/** \brief SCOP error code */
#define SCOP_UNKNOWN_ERRCODE                1
/** \brief SCOP error code */
#define SCOP_EXTERNAL_ERRCODE               2
/** \brief SCOP error code */
#define SCOP_DECODE_BADLENGTH_ERRCODE       10
/** \brief SCOP error code */
#define SCOP_DECODE_TRUNCMSG_ERRCODE        11
/** \brief SCOP error code */
#define SCOP_DECODE_NOTSCOP_ERRCODE         12 
/** \brief SCOP error code */
#define SCOP_DECODE_BADSCOPVERSION_ERRCODE  13
/** \brief SCOP error code */
#define SCOP_DECODE_BADTYPE_ERRCODE         14 
/** \brief SCOP error code */
#define SCOP_DECODE_BADPARAM_ERRCODE        15 
/** \brief SCOP error code */
#define SCOP_DECODE_BADVALUE_ERRCODE        16 
/** \brief SCOP error code */
#define SCOP_DECODE_PARAMREPEAT_ERRCODE     17
/** \brief SCOP error code */
#define SCOP_DECODE_MISSINGPARAM_ERRCODE    18
/** \brief SCOP error code */
#define SCOP_BADCMD_ERRCODE                 19
/** \brief SCOP error code */
#define SCOP_DECODE_EOM                     20


/* some defs */
/** \brief Folder where TC config files are stored */
#define SCOP_CONFIG_DIR        "./config"
/** \brief UTS config file */
#define SCOP_CONFIG_FILE       "./uts.conf"

/** \brief IPv4 SCOP Transport binding address */
#define SCOP_SERVER_ADDR       "172.30.8.4"
/** \brief IPv4 SCOP Transport binding port */
#define SCOP_SERVER_PORT       3751          /* ETSI ;-) */

/** \brief RS-232 SCOP Transport device */
#ifdef __linux__
#define SCOP_RS232_DEVICE      "/dev/ttyS0"
#else
#define SCOP_RS232_DEVICE      "/dev/ttyd0"
#endif

#ifdef __linux__
int      snprintf(char *, size_t, const char *, ...);
#endif /* __linux__ */



/**
 * \fn      void scopRun();
 * \brief   Start the UTS
 */
void scopRun();

/**
 * \fn      void scopTransportInit();
 * \brief   Initialize the SCOP Transport module
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopTransportInit();

/**
 * \fn      void scopTransportQuit();
 * \brief   Quit properly the SCOP Transport module
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopTransportQuit();

/**
 * \fn      void scopTransportSynchro(char * magic);
 * \brief   Go on reading data from transport layer until reading synchronization token
 *
 * \param   magic synchronization token
 */
void scopTransportSynchro(char * magic);

/**
 * \fn      void scopTransportRead(char *buffer, int len);
 * \brief   Retrieve bytes from the SCOP Transport layer  
 *
 * \param   buffer buffer where retrieved bytes are stored (must be malloc'd)
 * \param   len number of bytes to retrieve
 */
void scopTransportRead(char *buffer, int len);

/**
 * \fn      void scopTransportSend(char * buffer, int len);
 * \brief   Send message to the SCOP Transport layer
 *
 * \param   buffer pointer to SCOP message
 * \param   len size of the message
 */
void scopTransportSend(char * buffer, int len);

/**
 * \fn      int scopDecode();
 * \brief   Make the Decode module proceed the next pending message
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDecode();

/**
 * \fn      int scopDecodeInitIutRequest();
 * \brief   Proceed the current SCOP message as an InitRequest
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDecodeInitIutRequest();

/**
 * \fn      int scopDecodeResetRequest();
 * \brief   Proceed the current SCOP message as a ResetRequest
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDecodeResetRequest();

/**
 * \fn      int scopDecodeDataRequest();
 * \brief   Proceed the current SCOP message as a DataRequest 
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDecodeDataRequest();

/**
 * \fn      int scopExtractHeader(char **string);
 * \brief   Retrieve the header part of a SCOP message from Transport layer
 *
 * \param   string address of the pointer where the header will be stored 
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopExtractHeader(char **string);

/**
 * \fn      int scopExtractShortString(char **string);
 * \brief   Retrieve short string from Transport layer
 *
 * \param   string address of the pointer where the string will be stored 
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopExtractShortString(char **string);

/**
 * \fn      int scopExtractLongString(char **string);
 * \brief   Retrieve long string from Transport layer
 *
 * \param   string address of the pointer where the string will be stored 
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopExtractLongString(char **string);

/**
 * \fn      int scopExtractList();
 * \brief   Retrieve parameter list from Transport layer
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopExtractList();

/**
 * \fn      int scopEncodeInt(int value, char **result);
 * \brief   Encode integer into SCOP format
 *
 * \param   value integer to be encoded
 * \param   result address of the pointer where the result byte array will be stored 
 *
 * \warning If error occurs, result is set to NULL
 */
void scopEncodeInt(int value, char **result);

/**
 * \fn      int scopEncodeShortString(char *string, char **result);
 * \brief   Encode short string into SCOP format
 *
 * \param   string string to be encoded
 * \param   result address of the pointer where the result byte array will be stored 
 *
 * \warning If error occurs, result is set to NULL
 */
void scopEncodeShortString(char *string, char **result);

/**
 * \fn      int scopEncodeLongString(char *string, char **result);
 * \brief   Encode long string into SCOP format
 *
 * \param   string string to be encoded
 * \param   result address of the pointer where the result byte array will be stored 
 *
 * \warning If error occurs, result is set to NULL
 */
void scopEncodeLongString(char *string, char **result);

/**
 * \fn      int scopResponse(int code, char *desc);
 * \brief   Send a scopResponse to UTC
 *
 * \param   code SCOP response code
 * \param   desc SCOP error description
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopResponse(int code, char *desc);

/**
 * \fn      int scopDataResponse(int code, char *data);
 * \brief   Send a scopResponse to UTC
 *
 * \param   code SCOP rsponse code
 * \param   data expected data
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDataResponse(int code, char *data);

/**
 * \fn      int scopInitRequest(char *tc_id);
 * \brief   Proceed a scopInitrequest
 *
 * \param   tc_id Test Case identifier
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopInitRequest(char *tc_id);

/**
 * \fn      int scopResetRequest();
 * \brief   Proceed a scopResetRequest
 *
 * \return 0 if successful
 * \return 1+ if error occurs
 */
int scopResetRequest();

/**
 * \fn      int scopDataRequest(char *cmd, unsigned char *data);
 * \brief   Proceed a scopDataRequest
 *
 * \param   cmd type of scopDataResquest
 * \param   data pointer to data buffer
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDataRequest(char *cmd, unsigned char *data);

/**
 * \fn      int scopDisableIfaces();
 * \brief   Disable all network interfaces listed in UTS config file
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopDisableIfaces();

/**
 * \fn      int scopIfaceInit(char *tag);
 * \brief   Activate an interface using XML-like configuration string
 *
 * \param   tag XML-like configuration string
 *
 * \return  0 if successful
 * \return  1+ if error occurs
 */
int scopIfaceInit(char *tag);

