/*
 * ----------------------------------------------------------------------------
 *  (C) Copyright Testing Technologies, 2001-2006.  All Rights Reserved.
 *
 *  All copies of this program, whether in whole or in part, and whether
 *  modified or not, must display this and all other embedded copyright
 *  and ownership notices in full.
 *
 *  See the file COPYRIGHT for details of redistribution and use.
 *
 *  You should have received a copy of the COPYRIGHT file along with
 *  this file; if not, write to the Testing Technologies,
 *  Rosenthaler Str. 13, 10119 Berlin, Germany.
 *
 *  TESTING TECHNOLOGIES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *  SOFTWARE. IN NO EVENT SHALL TESTING TECHNOLOGIES BE LIABLE FOR ANY
 *  SPECIAL, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 *  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 *  THIS SOFTWARE.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 *  EITHER EXPRESSED OR IMPLIED, INCLUDING ANY KIND OF IMPLIED OR
 *  EXPRESSED WARRANTY OF NON-INFRINGEMENT OR THE IMPLIED WARRANTIES
 *  OF MECHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * ----------------------------------------------------------------------------
 *
 *  AUTHOR:      Dirk Tepelmann
 *  DATE:        November, 2005
 *
 *  REVISION INFO:
 *        $Revision$ $Date$
 * ----------------------------------------------------------------------------- */
package com.testingtech.ttcn.tci.codec.auth;

import org.etsi.ttcn.tci.RecordValue;

import com.testingtech.ttcn.tci.codec.tabular.TabularDecoder;
import com.testingtech.ttcn.tci.codec.tabular.TabularException;
import com.testingtech.ttcn.tci.codec.tabular.TabularObject;
import com.testingtech.ttcn.tci.codec.tabular.TabularOctetstring;
import com.testingtech.ttcn.tci.codec.tabular.TabularRecord;
import com.testingtech.ttcn.tri.IPv6TestAdapter;
import com.testingtech.ttcn.tri.enums.IEnumDefinitions;

import de.tu_berlin.cs.uebb.muttcn.runtime.InvalidAssignmentError;

public class AuthHeaderCodec implements IEnumDefinitions{
	
    /**
     * @param dec
     * @param to
     * @throws TabularException
     */
	public static void decodeAuthHeader(TabularDecoder dec, TabularObject to) throws TabularException {

		try {
			RecordValue rv = (RecordValue)((TabularRecord)to).tciValue;

			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 nextHeader;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 payloadLen;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16 reserved;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32 securityParametersIndex;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32 sequenceNumber;
			TabularOctetstring icv = null;
			TabularOctetstring icvPadding = null;

			// -- nextHeader
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8.isNext(
					dec)) {
				nextHeader = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
						"nextHeader"));
				nextHeader.decode(dec);
			} else {
				throw new TabularException(
						"Mandatory field 'nextHeader' of 'AuthHeader' missing");
			}
			rv.setField("nextHeader", nextHeader.toValue());

			dec.tag("nextHeader", nextHeader);

			// -- payloadLen
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8.isNext(
					dec)) {
				payloadLen = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
						"payloadLen"));
				payloadLen.decode(dec);
			} else {
				throw new TabularException(
						"Mandatory field 'payloadLen' of 'AuthHeader' missing");
			}
			rv.setField("payloadLen", payloadLen.toValue());

			dec.tag("authPayloadLen", payloadLen);
			dec.intTag("IPv6Header_payloadLength",
					dec.getIntTag("IPv6Header_payloadLength") -
					((TabularObject.valueOf(payloadLen) - 1) * 4) - 12);

			// -- reserved
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16.isNext(
					dec)) {
				reserved = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16(rv.getField(
						"reserved"));
				reserved.decode(dec);
			} else {
				throw new TabularException(
						"Mandatory field 'reserved' of 'AuthHeader' missing");
			}
			rv.setField("reserved", reserved.toValue());

			// -- securityParametersIndex
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32.isNext(
					dec)) {
				securityParametersIndex = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32(rv.getField(
						"securityParametersIndex"));
				securityParametersIndex.decode(dec);
			} else {
				throw new TabularException(
						"Mandatory field 'securityParametersIndex' of 'AuthHeader' missing");
			}
			rv.setField("securityParametersIndex", securityParametersIndex.toValue());

			// -- sequenceNumber
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32.isNext(
					dec)) {
				sequenceNumber = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32(rv.getField(
						"sequenceNumber"));
				sequenceNumber.decode(dec);
			} else {
				throw new TabularException(
						"Mandatory field 'sequenceNumber' of 'AuthHeader' missing");
			}
			rv.setField("sequenceNumber", sequenceNumber.toValue());

//			final RecordValue rvf = rv;

			final int icvLen = ((IPv6TestAdapter)dec.getRb().TestAdapter).getAhIcvLen(securityParametersIndex.toBigInt());

			// -- icv
			if (TabularOctetstring.isNext(dec)) {
				icv = (TabularOctetstring) new TabularOctetstring(rv.getField("icv")) {
					public int getLength() throws TabularException {
//						return getICVLen(rvf, dec.getRb());
						return icvLen;
					}
				}.decode(dec);
			}
//			if (getICVLen(rvf, dec.getRb()) != 0) {
			if (icvLen != 0) {
				rv.setField("icv", icv.toValue());
			}

			final int icvPaddingLength = (TabularObject.valueOf(dec.getTag("authPayloadLen")) - 1) * 4 - icv.length;
			// -- icvPadding
			if (TabularOctetstring.isNext(dec)) {
				icvPadding = (TabularOctetstring) new TabularOctetstring(rv.getField(
						"icvPadding")) {
					public int getLength() throws TabularException {
						return icvPaddingLength;
					}
				}.decode(dec);
			}
			if (icvPaddingLength>0) {
				rv.setField("icvPadding", icvPadding.toValue());
			}

		} catch (InvalidAssignmentError e) {
			throw new TabularException(e.getMessage());
		}		
	}

//	private static int getICVLen(RecordValue rv, RB rb) throws TabularException {
//		int spi = ((IntegerValue)rv.getField("securityParametersIndex")).getInt();
//		EnumeratedValue espIntegrityAlgo = ((IPv6TestAdapter)rb.TestAdapter).getEspIntegrityAlgo(spi);
//		if (espIntegrityAlgo==null) {
//			rb.getTciTLProvided().tliRT(
//					"",
//					System.nanoTime(),
//					null,
//					-1,
//					null,
//					RTLoggingConstants.RT_LOG_ERROR,
//					"No IntegrityAlgo saved for SPI " + spi
//							+ " retrieved from received IPv6 packet.");
//			return 0;
//		}
//		String algo = espIntegrityAlgo.getEnum();
//		if (algo != null) {
//			if (algo.equals(INTEGRITY_E_AUTH_NULL)) {
//				//no ICV
//				return 0;
//			}
//			else if (algo.equals(INTEGRITY_E_AUTH_HMACSHA1_96)) {
//				return 12;
//			}
//			else if (algo.equals(INTEGRITY_E_AUTH_AESXCBC_MAC_96)) {
//				return 12;
//			}
//			else if (algo.equals(INTEGRITY_E_AUTH_HMACMD5_96)) {
//				return 12;
//			}
//			else if (algo.equals(INTEGRITY_E_AUTH_SHA1_96)) {
//				return 12;
//			}
//			else if (algo.equals(INTEGRITY_E_AUTH_HMACSHA1_64)) {
//				return 8;
//			}
//			else {
//				System.err.println("decodeESPHeader: Unknown algorithm: " + algo);
//				throw new TabularException("decodeESPHeader: Unknown algorithm: " + algo);
//			}
//		}
//		else {
//			throw new TabularException("decodeAuthHeader: Integrity algorithm not yet configured!");
//		}
//	}
}
