/*
 * ----------------------------------------------------------------------------
 *  (C) Copyright Testing Technologies, 2001-2007.  All Rights Reserved.
 *
 *  All copies of this program, whether in whole or in part, and whether
 *  modified or not, must display this and all other embedded copyright
 *  and ownership notices in full.
 *
 *  See the file COPYRIGHT for details of redistribution and use.
 *
 *  You should have received a copy of the COPYRIGHT file along with
 *  this file; if not, write to the Testing Technologies,
 *  Rosenthaler Str. 13, 10119 Berlin, Germany.
 *
 *  TESTING TECHNOLOGIES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *  SOFTWARE. IN NO EVENT SHALL TESTING TECHNOLOGIES BE LIABLE FOR ANY
 *  SPECIAL, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 *  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 *  THIS SOFTWARE.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 *  EITHER EXPRESSED OR IMPLIED, INCLUDING ANY KIND OF IMPLIED OR
 *  EXPRESSED WARRANTY OF NON-INFRINGEMENT OR THE IMPLIED WARRANTIES
 *  OF MECHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * ----------------------------------------------------------------------------
 *
 *  AUTHOR:      Dirk Tepelmann
 *  DATE:        September, 2005
 *
 *  REVISION INFO:
 *        $Revision$ $Date$
 * ----------------------------------------------------------------------------- */
package com.testingtech.ttcn.tci.codec.esp;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;

import org.etsi.ttcn.tci.EnumeratedValue;
import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tci.OctetstringValue;
import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.Value;

import com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255;
import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tci.codec.tabular.TabularDecoder;
import com.testingtech.ttcn.tci.codec.tabular.TabularEncoder;
import com.testingtech.ttcn.tci.codec.tabular.TabularException;
import com.testingtech.ttcn.tci.codec.tabular.TabularObject;
import com.testingtech.ttcn.tci.codec.tabular.TabularOctetstring;
import com.testingtech.ttcn.tci.codec.tabular.TabularRecord;
import com.testingtech.ttcn.tri.IPv6TestAdapter;
import com.testingtech.ttcn.tri.encrypt.Encryption;
import com.testingtech.ttcn.tri.enums.IEnumDefinitions;
import com.testingtech.ttcn.tri.extFunc.ExternalFunctions_IPv6;
import com.testingtech.util.BitArrayInputStream;
import com.testingtech.util.BitArrayOutputStream;

import de.tu_berlin.cs.uebb.muttcn.runtime.InvalidAssignmentError;
import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.OctetString;

public class EspHeaderCodec implements IEnumDefinitions{
	
    /**
     * @param dec
     * @param to
     * @throws TabularException
     */
	public static void decodeESPHeader(TabularDecoder dec, TabularObject to) throws TabularException {
		try {		
			RecordValue rv = (RecordValue)((TabularRecord)to).tciValue;

			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32 spi;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32 seqNbr;
			com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspPayload espPayload;
			com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255 padding;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 padLength;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 nextHeader;
			TabularOctetstring icv = null;

			RB rb = dec.getRb();
			TabularOctetstring o = new TabularOctetstring(rb.tciCDRequired.getOctetstring().newInstance());
			byte [] completeMessage = ((TabularOctetstring)o.decode(dec)).tValue;

			IPv6TestAdapter ta = ((IPv6TestAdapter)rb.TestAdapter);

			int idx = 0;

			// decode first spi due to we need it for knowing the algo
			// -- spi
			spi = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32(rv.getField(
			"spi"));
			if (completeMessage.length>=(idx + spi.getLength()/8)) {
				spi.tValue = new BigInteger(1, getBytes(spi.getLength()/8, completeMessage, idx));
				spi.length = spi.getLength();
				spi.valid = true;
			} else {
				throw new TabularException(
				"Mandatory field 'securityParametersIndex' of 'EspHeader' missing");
			}
			idx += spi.getLength()/8;
			rv.setField("spi", spi.toValue());

			if (!ta.isSPIstored(spi.toBigInt())) {
				throw new TabularException("ESPHeader: Received message includes unknown SPI!");
			}

			//check the integrity first
			// -- icv
			int icvLen = ta.getEspIcvLen(spi.toBigInt());

			handleICV(icvLen, icv, rv, rb, completeMessage, ta);

			//decode the unciphered stuff

			// -- seqNbr
			seqNbr = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32(rv.getField(
			"seqNbr"));
			if (completeMessage.length>=(idx + seqNbr.getLength()/8)) {
				seqNbr.tValue = new BigInteger(1, getBytes(seqNbr.getLength()/8, completeMessage, idx));
				seqNbr.length = seqNbr.getLength();
				seqNbr.valid = true;
			}
			else {
				throw new TabularException(
				"Mandatory field 'sequenceNumber' of 'EspHeader' missing");
			}
			idx +=seqNbr.getLength()/8;
			rv.setField("seqNbr", seqNbr.toValue());

			// -- espPayload
			espPayload = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspPayload) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspPayload(rv.getField(
			"espPayload"));

			//try to fill ESP payload data
			RecordValue ep = (RecordValue)rb.getTciCDRequired().getTypeForName("LibIpv6_Interface_TypesAndValues.EspPayload").newInstance();
//			espPayload.plaintextData = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularPlaintextData) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularPlaintextData(pd);

			// -- iv
			EnumeratedValue espEncryptionAlgo = ta.getEspEncryptionAlgo(spi.toBigInt());
//			int ivLen = getIVlen(espEncryptionAlgo);
			int ivLen = 0;
			OctetString espIv = ta.getEspIv(spi.toBigInt());
			if (!espIv.notPresent()) {
				ivLen = espIv.getLength();
			}
			else {
				ivLen = getIVlen(espEncryptionAlgo);
			}

			if (ivLen>0) {
				espPayload.iv = getOctetstring(rb, completeMessage, ivLen, 8);
				ep.setField("iv", espPayload.iv.toValue());
			}
			idx += ivLen;

			//decode now the ciphered stuff

			byte [] payload = getBytes(completeMessage.length - idx - icvLen, completeMessage, idx);
			if ((espEncryptionAlgo!=null)&&!espEncryptionAlgo.getEnum().equals(ENCRYPTION_E_NULL)) {
				OctetString espEncKeyOs = ((OctetString)ta.getEspEncryptionKey(spi.toBigInt()));
				byte[] espEncKeyBa = null;
				if (espEncKeyOs != null)  {
					espEncKeyBa = espEncKeyOs.toByteArray();
				}
				String espEncAlgoString = null;
				if (espEncryptionAlgo!=null) {
					espEncAlgoString = espEncryptionAlgo.getEnum();
				}
				payload = Encryption.getDecryptedString(rb, espEncAlgoString, ta.getEspEncryptionKeyAlgo(spi.toBigInt()), espEncKeyBa, payload, espPayload.iv.tValue);
			}

			// -- nextHeader
			nextHeader = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
			"nextHeader"));
			int nextHdrLen = nextHeader.getLength()/8;
			int nextHdrIdx = payload.length - nextHdrLen;
			if (nextHdrIdx>0) {
				nextHeader.tValue = new BigInteger(1, getBytes(nextHdrLen, payload, nextHdrIdx)).intValue();
				nextHeader.length = nextHeader.getLength();
				nextHeader.valid = true;
			}
			else {
				throw new TabularException("Mandatory field 'nextHeader' of 'EspHeader' missing");
			}
			rv.setField("nextHeader", nextHeader.toValue());
			dec.tag("nextHeader", nextHeader);

			// -- padLength
			padLength = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
			"padLength"));
			int padLen = padLength.getLength()/8;
			int padLenIdx = payload.length - nextHdrLen - padLen;
			if (padLenIdx>0) {
				padLength.tValue = new BigInteger(1, getBytes(padLen, payload, padLenIdx)).intValue();
				padLength.length = padLength.getLength();
				padLength.valid = true;
			} else {
				throw new TabularException(
				"Mandatory field 'padLength' of 'EspHeader' missing");
			}
			rv.setField("padLength", padLength.toValue());

			// -- padding
			if (padLength.tValue > 0) {
				if (padLength.tValue <=255) {
					padding = (com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255) new com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255(rv.getField(
					"padding"));
					int paddingIdx = payload.length - nextHdrLen - padLen - padLength.tValue;
					if (paddingIdx>0) {
						padding.tValue = getBytes(padLength.tValue, payload, paddingIdx);
						padding.length = padLength.getLength();
					}
					else {
						throw new TabularException(
						"decodeESPHeader: Invalid message - invalid length!");
					}
					rv.setField("padding", padding.toValue());
				}
				else {
					throw new TabularException(
					"decodeESPHeader: Invalid padding length!");
				}
			}

			// -- espIpDatagram
			//cut padding and nextHeader
			payload = getBytes(payload.length - nextHdrLen - padLen - padLength.tValue, payload, 0);
//			int payloadLength = 0;
			TabularDecoder decPaylaod = new TabularDecoder(new BitArrayInputStream(payload), rb);
			decPaylaod.setTagMap(dec.getTagMap());
			decPaylaod.intTag("IPv6Header_payloadLength", payload.length);
			RecordValue eid = (RecordValue)rb.getTciCDRequired().getTypeForName("LibIpv6_Interface_TypesAndValues.EspIpDatagram").newInstance();
			espPayload.espIpDatagram = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspIpDatagram) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspIpDatagram(eid);
			espPayload.espIpDatagram.decode(decPaylaod);

			//TODO check if there is a better possibility
//			TabularEncoder enc = new TabularEncoder(new BitArrayOutputStream());
//			if (espPayload.espIpDatagram.ipv6Payload != null) {
//			espPayload.espIpDatagram.ipv6Payload.toValue();
//			espPayload.espIpDatagram.ipv6Payload.encode(enc);
//			}
//			payloadLength = decPaylaod.getIntTag("IPv6Header_payloadLength") - enc.toByteArray().length;
			ep.setField("espIpDatagram", espPayload.espIpDatagram.toValue());

			// -- tfcPadding
			espPayload.tfcPadding = (com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255) new com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255(ep.getField(
			"tfcPadding"));
			espPayload.tfcPadding.decode(decPaylaod);
			if (!espPayload.tfcPadding.tciValue.notPresent()) {
				ep.setField("tfcPadding", espPayload.tfcPadding.toValue());
			}
			else {
				espPayload.tfcPadding = null;
			}

			rv.setField("espPayload", espPayload.toValue());
		} catch (InvalidAssignmentError e) {
			throw new TabularException(e.getMessage());
		}

	}

    /**
     * @param enc
     * @param tciValue
     * @throws TabularException
     */
    public static void encodeESPHeader(TabularEncoder enc, Value tciValue) throws TabularException {
    	RecordValue rv = (RecordValue)tciValue;

        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32 spi;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32 seqNbr;
        com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspPayload espPayload;
        com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255 padding;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 padLength;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 nextHeader;
        TabularOctetstring icv;

		RB rb = enc.getRb();
		IPv6TestAdapter ta = ((IPv6TestAdapter)rb.TestAdapter);

		//encode all ESP to a temporary encode stream
		TabularEncoder encESP = new TabularEncoder(new BitArrayOutputStream());

		// -- spi
		spi = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32(rv.getField(
		"spi"));
		spi.encode(encESP);
		
		// -- seqNbr
		seqNbr = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt32(rv.getField(
		"seqNbr"));
		seqNbr.encode(encESP);

		// -- espPayload
		espPayload = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspPayload) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspPayload(rv.getField(
		"espPayload"));
		
		//Do we have to encrypt?
//		UnionValue ep = (UnionValue)rv.getField("espPayload");
//		if (ep.getPresentVariantName().equals("ciphertextData")) {
//			//No
//			espPayload.ciphertextData = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularCiphertextData) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularCiphertextData(ep.getVariant("ciphertextData"));
//			// -- iv
//			Value iv = ((RecordValue)espPayload.ciphertextData.tciValue).getField("iv");
//			if (!iv.notPresent()) {
//				espPayload.ciphertextData.iv = (TabularOctetstring) new TabularOctetstring(iv);
//				enc.encodeOctetstring(((OctetString)espPayload.ciphertextData.iv.tciValue).toByteArray());
//			}
//			espPayload.ciphertextData.ciphertextData = (TabularOctetstring) new TabularOctetstring(((RecordValue)espPayload.ciphertextData.tciValue).getField("ciphertextData"));
//			espPayload.ciphertextData.ciphertextData.encode(enc);
//		}
//		else if (ep.getPresentVariantName().equals("plaintextData")) {
			//Yes
//			espPayload.plaintextData = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularPlaintextData) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularPlaintextData(ep.getVariant("plaintextData"));
			// -- iv
		    boolean ivEncoded = false; 
			Value iv = ((RecordValue)espPayload.tciValue).getField("iv");
			espPayload.iv = (TabularOctetstring) new TabularOctetstring(iv);
			if (!iv.notPresent()) {
				encESP.encodeOctetstring(((OctetString)espPayload.iv.tciValue).toByteArray());
				espPayload.iv.tValue = ((OctetString)espPayload.iv.tciValue).toByteArray();
				ivEncoded = true;
			}

			TabularEncoder encPayload = new TabularEncoder(new BitArrayOutputStream()); 
			// -- espIpDatagram
			espPayload.espIpDatagram = (com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspIpDatagram) new com.testingtech.generated.codec.LibIpv6_Interface_TypesAndValues.TabularEspIpDatagram(((RecordValue)espPayload.tciValue).getField("espIpDatagram"));
			espPayload.espIpDatagram.encode(encPayload);
			
			// -- padding
			boolean calcPad = true;
			OctetstringValue paddingValue = (OctetstringValue)rv.getField("padding");
			IntegerValue padLengthValue = (IntegerValue)rv.getField("padLength");

			if (padLengthValue.getInt()!=0) {
				if (padLengthValue.getInt() == paddingValue.getLength()) {
					calcPad = false;
					padding = (TabularOct1to255) new TabularOct1to255(paddingValue);
					padding.encode(encPayload);
					
					// -- padLength
					padLength = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(padLengthValue);
					padLength.encode(encPayload);
				}
				else {
					String warning = "encodeESPHeader: Mismatch of padding and padLength! Replacing padding!";
					System.out.println(warning);
					rb.getTciTLProvided().tliRT(null, System.nanoTime(), null, 0, null, RTLoggingConstants.RT_LOG_WARN, warning);
				}
			}
			
			if (calcPad) {
				handlePadding(ta.getEspEncrBlockSize(spi.toBigInt()), encPayload);
			}
			
			// -- nextHeader
			nextHeader = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField("nextHeader"));
			nextHeader.encode(encPayload);
			
			byte [] payload = encPayload.toByteArray();
			if (!ta.getEspEncryptionAlgo(spi.toBigInt()).getEnum().equals(ENCRYPTION_E_NULL)) {
				OctetString espIv = ta.getEspIv(spi.toBigInt());
				if (!ivEncoded && !espIv.notPresent()) {
					rb.getTciTLProvided()
						.tliRT("", System.nanoTime(), "", -1, null,
								RTLoggingConstants.RT_LOG_WARN,
								"No IV specified in the message. Using the one from SA!");
					encESP.encodeOctetstring(espIv.toByteArray());
					espPayload.iv.tValue = espIv.toByteArray();
					ivEncoded = true;
					
				}

				ByteArrayOutputStream ivBaos = new ByteArrayOutputStream();
				if (espPayload.iv.tValue != null) {
					try {
						ivBaos.write(espPayload.iv.tValue);
					} catch (IOException e) {
						rb.tciTMProvided.tciError(e.getMessage());
					}
				}
				
				payload = Encryption.getEncryptedString(
						rb, ta.getEspEncryptionAlgo(spi.toBigInt()).getEnum(),
						ta.getEspEncryptionKeyAlgo(spi.toBigInt()),
						((OctetString) ta.getEspEncryptionKey(spi.toBigInt())).toByteArray(),
						payload, ivBaos);
				
				if (!ivEncoded && (ivBaos.size()>0)) {
					espPayload.iv.tValue = ivBaos.toByteArray();
					encESP.encodeOctetstring(espPayload.iv.tValue);
					ivEncoded = true;
				}
			}
			
			encESP.encodeOctetstring(payload);
//		}
//		else {
//			System.err.println("encodeESPHeader: Unkown variant for espPaylod " + ep.getPresentVariantName());
//			return;
//		}
		
		// -- ICV
		if (!rv.getField("icv").notPresent()) {
			icv = (TabularOctetstring) new TabularOctetstring(rv.getField("icv"));
			icv.encode(encESP);
		}
		else {
			if (!ta.getEspIntegrityAlgo(spi.toBigInt()).getEnum().equals(INTEGRITY_E_AUTH_NULL)) {
				ExternalFunctions_IPv6 efIPv6 = new ExternalFunctions_IPv6(rb);
				
				byte [] msg = encESP.toByteArray();
				OctetString icvO = (OctetString) efIPv6.fx_mac(
						ta.getEspIntegrityAlgo(spi.toBigInt()), 
						ta.getEspIntegrityKey(spi.toBigInt()), 
						new OctetString(rb, msg.length, msg)
				);
				encESP.encodeOctetstring(icvO.toByteArray());
			}
		}
		
		//finally encode to the output encode stream
		enc.encodeOctetstring(encESP.toByteArray());		
    }
    
	/**
	 * @param icvLen
	 * @param icv
	 * @param rv
	 * @param rb
	 * @param completeMessage
	 * @param ta
	 * @throws TabularException
	 */
//	private static int handleICV(TabularOctetstring icv, RecordValue rv, RB rb, byte[] completeMessage, IPv6TestAdapter ta) throws TabularException {
	private static void handleICV(int icvLen, TabularOctetstring icv, RecordValue rv, RB rb, byte[] completeMessage, IPv6TestAdapter ta) throws TabularException {
//		int icvLen = 0;
		BigInteger spi = ((IntegerValue)rv.getField("spi")).getBigInt();
		EnumeratedValue espIntegrityAlgo = ta.getEspIntegrityAlgo(spi);
		if (espIntegrityAlgo==null) {
//			return 0;
			return;
		}
		String algo = espIntegrityAlgo.getEnum();
		if (algo != null) {
			if (algo.equals(INTEGRITY_E_AUTH_NULL)) {
				//no ICV
//				return icvLen;
				return;
			}
			else if (algo.equals(INTEGRITY_E_AUTH_HMACSHA1_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity(spi, (char)0, "HmacSHA1", 96, icv, rb, completeMessage, ta)) {
					rv.setField("icv", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_AESXCBC_MAC_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				throw new TabularException("decodeESPHeader: IntegrityAlgo_E_AES_XCBC_MAC_96 not yet implemented!");
//				if (checkIntegrity((char)0, "AesXCBCMac", 96, icv, rb, completeMessage, ta)) {
//					rv.setField("icv", icv.toValue());
//				}
//				else {
//					throw new TabularException("Integrity check failed!");
//				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_HMACMD5_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity(spi, (char)0, "HmacMD5", 96, icv, rb, completeMessage, ta)) {
					rv.setField("icv", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_SHA1_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity(spi, (char)0, "SHA1", 96, icv, rb, completeMessage, ta)) {
					rv.setField("icv", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_HMACSHA1_64)) {
//				icvLen = 8;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity(spi, (char)0, "HmacSHA1", 96, icv, rb, completeMessage, ta)) {
					rv.setField("icv", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else {
				System.err.println("decodeESPHeader: Unknown algorithm: " + algo);
				throw new TabularException("decodeESPHeader: Unknown algorithm: " + algo);
			}
		}
		else {
			throw new TabularException("decodeESPHeader: Integrity algorithm not yet configured!");
		}
//		return icvLen;
	}

	/**
	 * @param algo
	 * @return
	 * @throws TabularException
	 */
	private static int getIVlen(EnumeratedValue encAlgo) throws TabularException {
		if (encAlgo != null) {
			String algo = encAlgo.getEnum();
			if (algo.equals(ENCRYPTION_E_NULL)) {
				//no IV
				return 0;
			}
			if (algo.equals(ENCRYPTION_E_TRIPLEDES_CBC)) {
				return 8;
			}
			else if (algo.equals(ENCRYPTION_E_AES_CBC)) {
				return 16;
			}
			else if (algo.equals(ENCRYPTION_E_AES_CTR)) {
				return 8;
			}
			else if (algo.equals(ENCRYPTION_E_DES_CBC)) {
				return 8;
			}
		}
		return 0;
	}

	/**
	 * @param blockSize
	 * @param enc
	 * @throws TabularException
	 */
	private static void handlePadding(int blockSize, TabularEncoder enc) throws TabularException {
//	private static void handlePadding(String algo, TabularEncoder enc) throws TabularException {
//		int blockSize = 4;
//		if (algo != null) {
//			if (algo.equals(ENCRYPTION_E_NULL)) {
//				//finally it's 1, however I just say 4
//				blockSize = 4;
//			}
//			else if (algo.equals(ENCRYPTION_E_TRIPLEDES_CBC)) {
//				blockSize = 8;
//			}
//			else if (algo.equals(ENCRYPTION_E_AES_CBC)) {
//				blockSize = 16;
//			}
//			else if (algo.equals(ENCRYPTION_E_AES_CTR)) {
//				blockSize = 16;
//			}
//			else if (algo.equals(ENCRYPTION_E_DES_CBC)) {
//				blockSize = 8;
//			}
//		}
//		else {
//			throw new TabularException("decodeESPHeader: Encryption algorithm not yet configured!");
//		}
		int len = blockSize-(enc.toByteArray().length+2)%blockSize;//2 - 1padLength+1nextHdr
		int cnt = 1;
		byte [] ba = new byte[len];
		for (int i=0; i<len; i++) {
			ba[i] = (byte)cnt++;
		}
		//add padding
		enc.encodeOctetstring(ba);
		//add padLength
		enc.encodeByte((byte)len);
	}

	/**
	 * @param len
	 * @param completeMessage
	 * @param offset
	 * @return
	 */
	private static byte[] getBytes(int len, byte[] completeMessage, int offset) {
		byte [] ba = new byte[len];
		System.arraycopy(completeMessage, offset, ba, 0, len);
		return ba;
	}

	/**
	 * @param spi
	 * @param algoType
	 * @param algoName
	 * @param algoLen
	 * @param icv
	 * @param rb
	 * @param completeMessage
	 * @param ta
	 * @return
	 * @throws TabularException
	 */
	private static boolean checkIntegrity(BigInteger spi, char algoType, String algoName, int algoLen, TabularOctetstring icv, RB rb, byte [] completeMessage, IPv6TestAdapter ta) throws TabularException {
		int i = completeMessage.length-icv.length;
		OctetString espWithoutICV = new OctetString(rb, i, getBytes(i, completeMessage, 0));
		OctetString os;
		switch (algoType) {
			case 0:
				os = (OctetString)Encryption.getEncryptHMACx(rb, algoName, ta.getEspIntegrityKey(spi), espWithoutICV, algoLen);
				break;
			case 1:
				os = (OctetString)Encryption.getSHA1Integrity(rb, espWithoutICV, algoLen);
				break;
			default:
				throw new TabularException("checkIntegrity: Unknown algorithm!");
		}
		OctetString icvCheck = new OctetString(rb, icv.length, icv.tValue);
		if (!os.equals(icvCheck)) {
			return false;
		}
		
		return true;
	}

	/**
	 * @param rb
	 * @param completeMessage
	 * @param octLen
	 * @param offset
	 * @return
	 */
	private static TabularOctetstring getOctetstring(RB rb, byte [] completeMessage, int octLen, int offset) {
		TabularOctetstring o = (TabularOctetstring) new TabularOctetstring(rb.tciCDRequired.getOctetstring().newInstance());
		o.length = octLen;
		o.tValue = getBytes(octLen, completeMessage, offset);
		
		return o;
	}
	
}
