/*
 * ----------------------------------------------------------------------------
 *  (C) Copyright Testing Technologies, 2001-2007.  All Rights Reserved.
 *
 *  All copies of this program, whether in whole or in part, and whether
 *  modified or not, must display this and all other embedded copyright
 *  and ownership notices in full.
 *
 *  See the file COPYRIGHT for details of redistribution and use.
 *
 *  You should have received a copy of the COPYRIGHT file along with
 *  this file; if not, write to the Testing Technologies,
 *  Rosenthaler Str. 13, 10119 Berlin, Germany.
 *
 *  TESTING TECHNOLOGIES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *  SOFTWARE. IN NO EVENT SHALL TESTING TECHNOLOGIES BE LIABLE FOR ANY
 *  SPECIAL, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 *  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 *  THIS SOFTWARE.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 *  EITHER EXPRESSED OR IMPLIED, INCLUDING ANY KIND OF IMPLIED OR
 *  EXPRESSED WARRANTY OF NON-INFRINGEMENT OR THE IMPLIED WARRANTIES
 *  OF MECHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * ----------------------------------------------------------------------------
 *
 *  AUTHOR:      Dirk Tepelmann
 *  DATE:        September, 2005
 *
 *  REVISION INFO:
 *        $Revision$ $Date$
 * ----------------------------------------------------------------------------- */
package com.testingtech.ttcn.tci.codec.ike;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.math.BigInteger;

import org.etsi.ttcn.tci.IntegerValue;
import org.etsi.ttcn.tci.OctetstringValue;
import org.etsi.ttcn.tci.RecordOfValue;
import org.etsi.ttcn.tci.RecordValue;
import org.etsi.ttcn.tci.Value;

import com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255;
import com.testingtech.ttcn.logging.RTLoggingConstants;
import com.testingtech.ttcn.tci.codec.tabular.TabularDecoder;
import com.testingtech.ttcn.tci.codec.tabular.TabularEncoder;
import com.testingtech.ttcn.tci.codec.tabular.TabularException;
import com.testingtech.ttcn.tci.codec.tabular.TabularObject;
import com.testingtech.ttcn.tci.codec.tabular.TabularOctetstring;
import com.testingtech.ttcn.tci.codec.tabular.TabularRecord;
import com.testingtech.ttcn.tri.IPv6TestAdapter;
import com.testingtech.ttcn.tri.encrypt.Encryption;
import com.testingtech.ttcn.tri.enums.IEnumDefinitions;
import com.testingtech.ttcn.tri.extFunc.ExternalFunctions_IPv6;
import com.testingtech.util.BitArrayInputStream;
import com.testingtech.util.BitArrayOutputStream;

import de.tu_berlin.cs.uebb.muttcn.runtime.InvalidAssignmentError;
import de.tu_berlin.cs.uebb.muttcn.runtime.RB;
import de.tu_berlin.cs.uebb.ttcn.runtime.OctetString;

public class EncryptedPayloadCodec implements IEnumDefinitions{
	
    /**
     * @param dec
     * @param to
     * @throws TabularException
     */
	public static void decodeEncryptedPayload(TabularDecoder dec, TabularObject to) throws TabularException {

		try {			
			RecordValue rv = (RecordValue)((TabularRecord)to).tciValue;

			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 nextPayload;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1 criticalFlag;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7 reserved;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16 payloadLength;
			TabularOctetstring iv = null;
			com.testingtech.generated.codec.LibIpv6_Rfc4306Ikev2_TypesAndValues.TabularIkePayloadList payloadList;
			TabularOctetstring padding;
			com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 padLength;
			TabularOctetstring integrityChecksumData = null;

			RB rb = dec.getRb();
			IPv6TestAdapter ta = ((IPv6TestAdapter)rb.TestAdapter);

			//decode the unciphered stuff

			// -- nextPayload
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8.isNext(dec)) {
				nextPayload = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
				"nextPayload"));
				nextPayload.decode(dec);
			} else {
				throw new TabularException(
				"Mandatory field 'nextPayload' of 'EncryptedPayload' missing");
			}
			rv.setField("nextPayload", nextPayload.toValue());

			dec.tag("nextPayload", nextPayload);

			// -- criticalFlag
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1.isNext(dec)) {
				criticalFlag = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1(rv.getField(
				"criticalFlag"));
				criticalFlag.decode(dec);
			}
			else {
				throw new TabularException(
				"Mandatory field 'criticalFlag' of 'EncryptedPayload' missing");
			}
			rv.setField("criticalFlag", criticalFlag.toValue());

			// -- reserved
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7.isNext(dec)) {
				reserved = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7(rv.getField(
				"reserved"));
				reserved.decode(dec);
			}
			else {
				throw new TabularException(
				"Mandatory field 'reserved' of 'EncryptedPayload' missing");
			}
			rv.setField("reserved", reserved.toValue());

			// -- payloadLength
			if (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16.isNext(dec)) {
				payloadLength = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16(rv.getField(
				"payloadLength"));
				payloadLength.decode(dec);
			} else {
				throw new TabularException(
				"Mandatory field 'payloadLength' of 'EncryptedPayload' missing");
			}
			rv.setField("payloadLength", payloadLength.toValue());

			final String ikeEncryptionAlgo = ta.getIkeEncryptionAlgo();
			// -- iv
			if (TabularOctetstring.isNext(dec)) {
				iv = (TabularOctetstring) new TabularOctetstring(rv.getField(
						"iv")) {
					public int getLength() throws TabularException {
						return getIVlen(ikeEncryptionAlgo);
					}
				}.decode(dec);
			}
			rv.setField("iv", iv.toValue());

			TabularOctetstring o = new TabularOctetstring(rb.tciCDRequired.getOctetstring().newInstance());
			byte [] markedBytes = dec.getBytesFromMark();
			byte [] encPayloadMessage = ((TabularOctetstring)o.decode(dec)).tValue;
			ByteArrayOutputStream completeMessage = new ByteArrayOutputStream();
			try {
				completeMessage.write(markedBytes);
				completeMessage.write(encPayloadMessage);
			} catch (IOException e) {
				dec.getRb().getTciTLProvided().tliRT(
						"", System.nanoTime(), "", -1,
						null, RTLoggingConstants.RT_LOG_ERROR, e.getMessage()
				);
				e.printStackTrace();
				return;
			}
			//check the integrity first
			// -- integrityChecksumData
//			int icvLen = handleICV(integrityChecksumData, rv, rb, completeMessage.toByteArray(), ta);
			int icvLen = ta.getIkeIcvLen();
			handleICV(icvLen, integrityChecksumData, rv, rb, completeMessage.toByteArray(), ta, (TabularObject.valueOf(dec.getTag("iFlag"))==1)?true:false);

			//decode now the ciphered stuff

			byte [] payload = getBytes(encPayloadMessage.length - icvLen, encPayloadMessage, 0);
			if ((ikeEncryptionAlgo!=null)&&!ikeEncryptionAlgo.equals(ENCRYPTION_E_NULL)) {
				OctetString ikeEncKeyOs = (OctetString)ta.getIkeEncryptionKey((TabularObject.valueOf(dec.getTag("iFlag"))==1)?true:false);
				byte[] ikeEncKeyBa = null;
				if (ikeEncKeyOs != null)  {
					ikeEncKeyBa = ikeEncKeyOs.toByteArray();
				}
				payload = Encryption.getDecryptedString(rb, ikeEncryptionAlgo, ta.getIkeEncryptionKeyAlgo(), ikeEncKeyBa, payload, iv.tValue);
			}

			// -- padLength
			padLength = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
			"padLength"));
			int padLen = padLength.getLength()/8;
			int padLenIdx = payload.length - padLen;
			if (padLenIdx>0) {
				padLength.tValue = new BigInteger(1, getBytes(padLen, payload, padLenIdx)).intValue();
				padLength.length = padLength.getLength();
				padLength.valid = true;
			} else {
				throw new TabularException(
				"Mandatory field 'padLength' of 'EncryptedPayload' missing");
			}
			rv.setField("padLength", padLength.toValue());

			// -- padding
			if (padLength.tValue > 0) {
				if (padLength.tValue <=255) {
					padding = (com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255) new com.testingtech.generated.codec.LibCommon_DataStrings.TabularOct1to255(rv.getField(
					"padding"));
					int paddingIdx = payload.length - padLen - padLength.tValue;
					if (paddingIdx>=0) {
						padding.tValue = getBytes(padLength.tValue, payload, paddingIdx);
						padding.length = padLength.getLength();
					}
					else {
						throw new TabularException(
						"decodeEncryptedPayload: Invalid message - invalid length!");
					}
					rv.setField("padding", padding.toValue());
				}
				else {
					throw new TabularException(
					"decodeEncryptedPayload: Invalid padding length!");
				}
			}

			// -- payloadList
			//cut padding and padLen
			RecordOfValue pll = (RecordOfValue)rb.getTciCDRequired().getTypeForName("LibIpv6_Rfc4306Ikev2_TypesAndValues.IkePayloadList").newInstance();
			payloadList = (com.testingtech.generated.codec.LibIpv6_Rfc4306Ikev2_TypesAndValues.TabularIkePayloadList) new com.testingtech.generated.codec.LibIpv6_Rfc4306Ikev2_TypesAndValues.TabularIkePayloadList(pll);
			if (payload.length - padLen - padLength.tValue > 0) {
				payload = getBytes(payload.length - padLen - padLength.tValue, payload, 0);
				TabularDecoder decPaylaod = new TabularDecoder(new BitArrayInputStream(payload), rb);
				decPaylaod.setTagMap(dec.getTagMap());
				payloadList.decode(decPaylaod);
			}
			rv.setField("payloadList", payloadList.toValue());
		} catch (InvalidAssignmentError e) {
			throw new TabularException(e.getMessage());
		}

	}

    /**
     * @param enc
     * @param tciValue
     * @throws TabularException
     */
    public static void encodeEncryptedPayload(TabularEncoder enc, Value tciValue) throws TabularException {
    	RecordValue rv = (RecordValue)tciValue;

        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 nextPayload;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1 criticalFlag;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7 reserved;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16 payloadLength;
        TabularOctetstring iv;
        com.testingtech.generated.codec.LibIpv6_Rfc4306Ikev2_TypesAndValues.TabularIkePayloadList payloadList;
        TabularOctetstring padding;
        com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8 padLength;
        TabularOctetstring integrityChecksumData;

		RB rb = enc.getRb();
		IPv6TestAdapter ta = ((IPv6TestAdapter)rb.TestAdapter);

		//encode all to a temporary encode stream
		TabularEncoder encIKE = new TabularEncoder(new BitArrayOutputStream());

		// -- nextPayload
		nextPayload = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(rv.getField(
		"nextPayload"));
		nextPayload.encode(encIKE);
		
		// -- criticalFlag
		criticalFlag = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt1(rv.getField(
		"criticalFlag"));
		criticalFlag.encode(encIKE);

		// -- reserved
		reserved = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt7(rv.getField(
		"reserved"));
		reserved.encode(encIKE);

		// -- payloadLength
		payloadLength = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt16(rv.getField(
		"payloadLength"));
		payloadLength.encode(encIKE);

		//Do we have to encrypt?
		// -- iv
		boolean ivEncoded = false;
		iv = (TabularOctetstring) new TabularOctetstring(rv.getField("iv"));
		if (!rv.getField("iv").notPresent()) {
			encIKE.encodeOctetstring(((OctetString)iv.tciValue).toByteArray());
			iv.tValue = ((OctetString)iv.tciValue).toByteArray();
			ivEncoded = true;
		}

		TabularEncoder encPayload = new TabularEncoder(new BitArrayOutputStream()); 
		// -- payloadList
		if (!rv.getField("payloadList").notPresent()) {
			payloadList = (com.testingtech.generated.codec.LibIpv6_Rfc4306Ikev2_TypesAndValues.TabularIkePayloadList) new com.testingtech.generated.codec.LibIpv6_Rfc4306Ikev2_TypesAndValues.TabularIkePayloadList(rv.getField("payloadList"));
			payloadList.encode(encPayload);
		}

		// -- padding
		boolean calcPad = true;
		OctetstringValue paddingValue = (OctetstringValue)rv.getField("padding");
		IntegerValue padLengthValue = (IntegerValue)rv.getField("padLength");

		if (padLengthValue.getInt()!=0) {
			if (padLengthValue.getInt() == paddingValue.getLength()) {
				calcPad = false;
				padding = (TabularOct1to255) new TabularOct1to255(paddingValue);
				padding.encode(encPayload);

				// -- padLength
				padLength = (com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8) new com.testingtech.generated.codec.LibCommon_BasicTypesAndValues.TabularUInt8(padLengthValue);
				padLength.encode(encPayload);
			}
			else {
				String warning = "encodeEncryptedPayload: Mismatch of padding and padLength! Replacing padding!";
				System.out.println(warning);
				rb.getTciTLProvided().tliRT(null, System.nanoTime(), null, 0, null, RTLoggingConstants.RT_LOG_WARN, warning);
			}
		}

		if (calcPad) {
//			handlePadding(ta.getIkeEncryptionAlgo(), encPayload);
			handlePadding(ta.getIkeEncrBlockSize(), encPayload);
		}

		byte [] markedBytes = enc.getBytesFromMark();

		boolean initiator = false;
		if (markedBytes.length>0) {
			initiator = ((markedBytes[19]&0x08)==0x08)?true:false;
		}
		
		byte [] payload = encPayload.toByteArray();
		if (!ta.getIkeEncryptionAlgo().equals(ENCRYPTION_E_NULL)) {
			ByteArrayOutputStream ivBaos = new ByteArrayOutputStream();
			if (iv.tValue != null) {
				try {
					ivBaos.write(iv.tValue);
				} catch (IOException e) {
					rb.tciTMProvided.tciError(e.getMessage());
				}
			}
			payload = Encryption.getEncryptedString(rb, ta.getIkeEncryptionAlgo(), ta.getIkeEncryptionKeyAlgo(), ((OctetString)ta.getIkeEncryptionKey(initiator)).toByteArray(), payload, ivBaos);
			if (!ivEncoded && (ivBaos.size()>0)) {
				iv.tValue = ivBaos.toByteArray();
				encIKE.encodeOctetstring(iv.tValue);
				ivEncoded = true;
			}
		}
		encIKE.encodeOctetstring(payload);
		
		// -- integrityChecksumData
		if (!rv.getField("integrityChecksumData").notPresent()) {
			integrityChecksumData = (TabularOctetstring) new TabularOctetstring(rv.getField("integrityChecksumData"));
			integrityChecksumData.encode(encIKE);
		}
		else {
			if (!ta.getIkeIntegrityAlgo().equals(INTEGRITY_E_NONE)) {
				ExternalFunctions_IPv6 efIPv6 = new ExternalFunctions_IPv6(rb);
				
				byte [] msg = encIKE.toByteArray();
				ByteArrayOutputStream icvBaos = new ByteArrayOutputStream();
				try {
					icvBaos.write(markedBytes);
					icvBaos.write(msg);
				} catch (IOException e) {
					enc.getRb().getTciTLProvided().tliRT(
							"", System.nanoTime(), "", -1,
							null, RTLoggingConstants.RT_LOG_ERROR, e.getMessage()
					);
					e.printStackTrace();
					return;
				}
				byte[] icvMsg = icvBaos.toByteArray();
				OctetString icvO = (OctetString)efIPv6.integrity(ta.getIkeIntegrityAlgo(), ta.getIkeIntegrityKey(initiator), new OctetString(rb, icvMsg.length, icvMsg));
				encIKE.encodeOctetstring(icvO.toByteArray());
			}
		}
		
		//finally encode to the output encode stream
		enc.encodeOctetstring(encIKE.toByteArray());		
    }
    
	/**
	 * @param icv
	 * @param rv
	 * @param rb
	 * @param completeMessage
	 * @param ta
	 * @return
	 * @throws TabularException
	 */
	private static void handleICV(int icvLen, TabularOctetstring icv, RecordValue rv, RB rb, byte[] completeMessage, IPv6TestAdapter ta, boolean initiator) throws TabularException {
//	private static int handleICV(TabularOctetstring icv, RecordValue rv, RB rb, byte[] completeMessage, IPv6TestAdapter ta) throws TabularException {
//		int icvLen = 0;
		String algo = ta.getIkeIntegrityAlgo();
		if (algo != null) {
			if (algo.equals(INTEGRITY_E_AUTH_NULL) || algo.equals(INTEGRITY_E_NONE)) {
				//no ICV
//				return icvLen;
				return;
			}
			else if (algo.equals(INTEGRITY_E_AUTH_HMACSHA1_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity((char)0, "HmacSHA1", 96, icv, rb, completeMessage, ta, initiator)) {
					rv.setField("integrityChecksumData", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_AESXCBC_MAC_96) || algo.equals(INTEGRITY_E_AUTH_AESXCBC_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				throw new TabularException("decodeEncryptedPayload: IntegrityAlgo_E_AES_XCBC_MAC_96 not yet implemented!");
//				if (checkIntegrity((char)0, "AesXCBCMac", 96, icv, rb, completeMessage, ta, initiator)) {
//					rv.setField("integrityChecksumData", icv.toValue());
//				}
//				else {
//					throw new TabularException("Integrity check failed!");
//				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_HMACMD5_96)) {
//				icvLen = 12;
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity((char)0, "HmacMD5", 96, icv, rb, completeMessage, ta, initiator)) {
					rv.setField("integrityChecksumData", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_SHA1_96)) {
//				icvLen = 0; //TODO unknown length
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity((char)1, "SHA1", 96, icv, rb, completeMessage, ta, initiator)) {
					rv.setField("integrityChecksumData", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_HMACSHA1_64)) {
//				icvLen = 0; //TODO unknown length
				icv = getOctetstring(rb, completeMessage, icvLen, completeMessage.length - icvLen);
				if (checkIntegrity((char)0, "HmacSHA1", 96, icv, rb, completeMessage, ta, initiator)) {
					rv.setField("integrityChecksumData", icv.toValue());
				}
				else {
					throw new TabularException("Integrity check failed!");
				}
			}
			else if (algo.equals(INTEGRITY_E_AUTH_KPDKMD5)) {
//				icvLen = 0; //TODO unknown length
			}
			else if (algo.equals(INTEGRITY_E_AUTH_AESXCBC_96)) {
				icvLen = 0; //TODO unknown length
			}
			else {
				System.err.println("decodeEncryptedPayload: Unknown algorithm: " + algo);
				throw new TabularException("decodeEncryptedPayload: Unknown algorithm: " + algo);
			}
		}
		else {
			throw new TabularException("decodeEncryptedPayload: Integrity algorithm not yet configured!");
		}
//		return icvLen;
	}

	/**
	 * @param algo
	 * @return
	 * @throws TabularException
	 */
	private static int getIVlen(String algo) throws TabularException {
		if (algo != null) {
			if (algo.equals(ENCRYPTION_E_NULL)) {
				//no IV
				return 0;
			}
			else if (algo.equals(ENCRYPTION_E_TRIPLEDES_CBC)) {
				return 8;
			}
			else if (algo.equals(ENCRYPTION_E_AES_CBC)) {
				return 16;
			}
			else if (algo.equals(ENCRYPTION_E_AES_CTR)) {
				return 8;
			}
			else if (algo.equals(ENCRYPTION_E_DES_CBC)) {
				return 8;
			}
		}
		return 0;
	}

	/**
	 * @param blockSize
	 * @param enc
	 * @throws TabularException
	 */
	private static void handlePadding(int blockSize, TabularEncoder enc) throws TabularException {
//	private static void handlePadding(String algo, TabularEncoder enc) throws TabularException {
//		int blockSize = 4;
//		if (algo != null) {
//			if (algo.equals(ENCRYPTION_E_NULL)) {
//				//finally it's 1, however I just say 4
//				blockSize = 4;
//			}
//			else if (algo.equals(ENCRYPTION_E_TRIPLEDES_CBC)) {
//				blockSize = 8;
//			}
//			else if (algo.equals(ENCRYPTION_E_AES_CBC)) {
//				blockSize = 16;
//			}
//			else if (algo.equals(ENCRYPTION_E_AES_CTR)) {
//				blockSize = 16;
//			}
//			else if (algo.equals(ENCRYPTION_E_DES_CBC)) {
//				blockSize = 8;
//			}
//		}
//		else {
//			throw new TabularException("decodeEncryptedPayload: Encryption algorithm not yet configured!");
//		}
		int len = blockSize-(enc.toByteArray().length+1)%blockSize;//1 - 1padLength
		int cnt = 1;
		byte [] ba = new byte[len];
		for (int i=0; i<len; i++) {
			ba[i] = (byte)cnt++;
		}
		//add padding
		enc.encodeOctetstring(ba);
		//add padLength
		enc.encodeByte((byte)len);
	}

	/**
	 * @param len
	 * @param completeMessage
	 * @param offset
	 * @return
	 */
	private static byte[] getBytes(int len, byte[] completeMessage, int offset) {
		byte [] ba = new byte[len];
		System.arraycopy(completeMessage, offset, ba, 0, len);
		return ba;
	}

	/**
	 * @param algoType
	 * @param algoName
	 * @param algoLen
	 * @param icv
	 * @param rb
	 * @param completeMessage
	 * @param ta
	 * @param initiator TODO
	 * @return
	 * @throws TabularException
	 */
	private static boolean checkIntegrity(char algoType, String algoName, int algoLen, TabularOctetstring icv, RB rb, byte [] completeMessage, IPv6TestAdapter ta, boolean initiator) throws TabularException {
		int i = completeMessage.length-icv.length;
		OctetString ikeWithoutICV = new OctetString(rb, i, getBytes(i, completeMessage, 0));
		OctetString os;
		switch (algoType) {
			case 0:
				os = (OctetString)Encryption.getEncryptHMACx(rb, algoName, ta.getIkeIntegrityKey(initiator), ikeWithoutICV, algoLen);
				break;
			case 1:
				os = (OctetString)Encryption.getSHA1Integrity(rb, ikeWithoutICV, algoLen);
				break;
			default:
				throw new TabularException("checkIntegrity: Unknown algorithm!");
		}
		OctetString icvCheck = new OctetString(rb, icv.length, icv.tValue);
		if (!os.equals(icvCheck)) {
			return false;
		}
		
		return true;
	}

	/**
	 * @param rb
	 * @param completeMessage
	 * @param octLen
	 * @param offset
	 * @return
	 */
	private static TabularOctetstring getOctetstring(RB rb, byte [] completeMessage, int octLen, int offset) {
		TabularOctetstring o = (TabularOctetstring) new TabularOctetstring(rb.tciCDRequired.getOctetstring().newInstance());
		o.length = octLen;
		o.tValue = getBytes(octLen, completeMessage, offset);
		
		return o;
	}
	
}
