/*
 * ----------------------------------------------------------------------------
 *  (C) Copyright Testing Technologies, 2001-2005.  All Rights Reserved.
 *
 *  All copies of this program, whether in whole or in part, and whether
 *  modified or not, must display this and all other embedded copyright
 *  and ownership notices in full.
 *
 *  See the file COPYRIGHT for details of redistribution and use.
 *
 *  You should have received a copy of the COPYRIGHT file along with
 *  this file; if not, write to the Testing Technologies,
 *  Rosenthaler Str. 13, 10119 Berlin, Germany.
 *
 *  TESTING TECHNOLOGIES DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
 *  SOFTWARE. IN NO EVENT SHALL TESTING TECHNOLOGIES BE LIABLE FOR ANY
 *  SPECIAL, DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 *  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
 *  AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
 *  ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF
 *  THIS SOFTWARE.
 *
 *  THE SOFTWARE IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
 *  EITHER EXPRESSED OR IMPLIED, INCLUDING ANY KIND OF IMPLIED OR
 *  EXPRESSED WARRANTY OF NON-INFRINGEMENT OR THE IMPLIED WARRANTIES
 *  OF MECHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 *
 * ----------------------------------------------------------------------------
 *
 *  AUTHOR:      Dirk Tepelmann
 *  DATE:        June 2005
 *
 *  REVISION INFO:
 *     $Revision: 1.1 $  $Date: 2005/06/22 14:03:44 $
 *     $Source: /net/ttech/cvs_root/IPv6_ETSI_STF276/javasrc/com/testingtech/ttcn/tri/TestAdapterDefinitions.java,v $
 *
 * ----------------------------------------------------------------------------
 */
package com.testingtech.ttcn.tri;

/**
 * This interface contains all the definition of the testadpater and 
 * was created to have one location for changing the definitions.
 * 
 * @author dte
 */
public interface TestAdapterDefinitions {

    /**
     * Constants declaration for field names of the record CfMessage
     */
	
    public final String CfgMessage			= "CfMessage";
    public final String[] IPv6NODEPARAMSLIST = {
    		"paramsHs01",
    		"paramsHs02",
    		"paramsHs03",
    		"paramsRt01",
    		"paramsRt02",
    		"paramsRt03",
    		"paramsRt04",
    		"paramsMn01",
    		"paramsIut"
    };
    
    public final String ROLE 				= "role";
//    	public final String HOST_ROLE 			= "Host";
    public final String ROUTER_ROLE 		= "Router";
    public final String IPV4ROUTER_ROLE 		= "Ipv4Router";
    public final String NETDEVICEID 		= "netDeviceId";
    public final String MACFILTERLIST 		= "macFilterList";
    
    public final String[][] IPv6NODEPARAMS = {
    		{ "lla", "macUca" },
    		{ "gla", "macUca" },
    		{ "solNodeMca", 	"macSolNodeMca" },
    		{ "haAca", "macUca" },
    		{ "mnHoa", "macUca" },
    		{ "mnCoa", "macUca" },
    		{ "ipv4Addr", "macUca" },
    		{ "relRtAca", "macUca" }
    };
    public final int FADDR = 0;
    public final int FADDRMAC = 1;
    public final String HANDLECFG	= "useInTa";
    
    /**
     * Well-known MAC addresses
     */
    
    public final byte[] MAC_ALLNODESMCA   = new byte[] {(byte)0x33, (byte)0x33, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x01};
    public final byte[] MAC_ALLROUTERSMCA = new byte[] {(byte)0x33, (byte)0x33, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x02};
    
    /**
     * Well-known IP addresses
     */
    
    public final byte[] IP6_ALLNODESMCA    = new byte[] {(byte)0xFF, (byte)0x02, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x01};
    public final byte[] IP6_ALLROUTERSMCA  = new byte[] {(byte)0xFF, (byte)0x02, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x02};
    public final byte[] IP6_SOLNODEMCA     = new byte[] {(byte)0xFF, (byte)0x02, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x01, (byte)0xFF};
    public final byte[] IP6_UNSPECIFIED    = new byte[] {(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00};

    public final short IPv6FRAMETYPE = (short)0x86DD;
    
    public final byte[] IP4_ALLNODESMCA    = new byte[] {(byte)0xEF, (byte)0xC0, (byte)0x00, (byte)0x01}; //239.192.0.1
    public final byte[] IP4_ALLROUTERSMCA  = new byte[] {(byte)0xEF, (byte)0xC0, (byte)0x00, (byte)0x02}; //239.192.0.2
    public final byte[] IP4_SOLNODEMCA     = new byte[] {/*TODO*/};
    public final byte[] IP4_UNSPECIFIED    = new byte[] {(byte)0xEF, (byte)0x7A, (byte)0x00, (byte)0x01}; //239.122.0.1

    public final short IPv4FRAMETYPE = (short)0x0800;
    
    public final short ARPFRAMETYPE = (short)0x0806;
    
    /**
     * definitions of ports
     */
    
    public final String CFGPORT      = "cfPort";
    public final String CFG_TA_PORT  = "cfTaPort";
    public final String IPPORT       = "ipPort";
    public final String IP4PORT      = "ipv4Port";
    public final String ARPPORT      = "arpPort";
    public final String COMPSCOPPORT     = "escopPort";
    public final String TSISCOPPORT    = "escopXPort";
    public final String ETHERNETPORT = "ethernetPort";
    public final String IPV4ETHERNETPORT = "ipv4EthernetPort";
    public final String ARPETHERNETPORT = "arpEthernetPort";
    
    /**
     * definition of module pars
     */
    public final String[] PX_LLA_X = {
    	    "PX_LLA_RT01_A", 
    	    "PX_LLA_RT01_B", 
    	    "PX_LLA_RT03_A", 
    	    "PX_LLA_RT03_C", 
    	    "PX_LLA_HS01", 
    	    "PX_LLA_HS02", 
    	    "PX_LLA_HS03",
    	    "PX_LLA_HOA_MN01"
    };
    
    public final String[] PX_MAC_UCA_X = {
    	    "PX_MAC_UCA_RT01_A", 
    	    "PX_MAC_UCA_RT01_B", 
    	    "PX_MAC_UCA_RT03_A", 
    	    "PX_MAC_UCA_RT03_C", 
    	    "PX_MAC_UCA_HS01", 
    	    "PX_MAC_UCA_HS02", 
    	    "PX_MAC_UCA_HS03",
    	    "PX_MAC_UCA_MN01"
    };
    
    public final String MODULEPAR_NOT_FOUND = "Could not find module parameter: ";
	
    /**
     * triSend problems
     */
    
    public final String TRISEND_ND_MAC_CONFIGURE_ERROR			= "netDeviceId/MAC is not configured adequately!(triSend)";
    public final String TRISEND_ND_CONFIGURE_ERROR 				= "netDeviceId is not configured adequately!(triSend)";
    public final String TRISEND_SRC_IPADDRESS_CONFIGURE_ERROR 	= " not yet configured(triSend)!";
    public final String TRISEND_DST_IPADDRESS_CONFIGURE_ERROR 	= " not yet configured(triSend)!";
    public final String TRISEND_JPCAPSENDER_ERROR 				= "Cannot send message due to incomplete testadapter configuration(triSend)";
    
    /**
     * triEnqueueMsg problems
     */
    
    public final String TSIPORTID_NOT_FOUND 					= "triEnqueueMsg: No tsiPortId mapped for ";
    public final String TRIENQUEUEMSG_MACADDRESS_NOT_CONFIGURED	= "triEnqueueMsg: MAC address not yet configured: ";
    public final String TRIENQUEUEMSG_NO_COMPONENT_FOUND		= "triEnqueueMsg: No componentId mapped for ipv6Address: ";
    
}
